import axios, { AxiosHeaders } from 'axios';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol, Iterator */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    if (typeof b !== "function" && b !== null)
        throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g = Object.create((typeof Iterator === "function" ? Iterator : Object).prototype);
    return g.next = verb(0), g["throw"] = verb(1), g["return"] = verb(2), typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

// 自定义错误类
var CustomError = /** @class */ (function (_super) {
    __extends(CustomError, _super);
    function CustomError(message) {
        var _this = _super.call(this, message) || this;
        _this.name = "CustomError";
        return _this;
    }
    return CustomError;
}(Error));
/**
 * AxiosFactory 类用于创建和配置 Axios 实例。
 * 处理登录认证并管理 cookies 和 tokens。
 */
var AxiosFactory = /** @class */ (function () {
    /**
     * 构造函数初始化 AxiosFactory。
     * @param config - AxiosFactory 的配置参数。
     * @param axiosConfig - Axios 的配置参数。
     * @param cache - 缓存配置（目前未使用）。
     */
    function AxiosFactory(config, axiosConfig, cache) {
        this.cookies = new Map();
        this.tokens = {};
        this.verified = false;
        this.maxCookies = 50; // 设置最大 cookie 数量
        this.config = config;
        this.axiosClient = this.createAxiosClient(axiosConfig);
    }
    /**
     * 初始化方法，通过调用登录方法进行认证。
     */
    AxiosFactory.prototype.initialize = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.login()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * 登录方法，发送登录请求并处理响应。
     */
    AxiosFactory.prototype.login = function () {
        return __awaiter(this, void 0, void 0, function () {
            var loginConfig, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        loginConfig = {
                            url: this.config.auth.uri,
                            method: "POST",
                            data: this.config.auth.data,
                        };
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.axiosClient(loginConfig)];
                    case 2:
                        _a.sent();
                        this.verified = true;
                        console.log("httpApp:".concat(this.config.name, "\u521D\u59CB\u5316\u6210\u529F"));
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        console.error("httpApp:".concat(this.config.name, "\u521D\u59CB\u5316\u5931\u8D25"), error_1);
                        throw new CustomError("Failed to initialize httpApp: ".concat(this.config.name));
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * 创建 Axios 实例并设置拦截器。
     * @param axiosConfig - Axios 的配置参数。
     * @returns 创建的 Axios 实例。
     */
    AxiosFactory.prototype.createAxiosClient = function (axiosConfig) {
        var config = __assign({ baseURL: "", withCredentials: true, timeout: 10000 }, axiosConfig);
        var api = axios.create(config);
        this.setupInterceptors(api);
        return api;
    };
    /**
     * 设置请求和响应拦截器。
     * @param api - Axios 实例。
     */
    AxiosFactory.prototype.setupInterceptors = function (api) {
        var _this = this;
        // 请求拦截器
        api.interceptors.request.use(function (config) {
            var headers = new AxiosHeaders();
            headers.set("cookie", Array.from(_this.cookies.values()).join(";"));
            if (_this.config.headers) {
                Object.entries(_this.config.headers).forEach(function (_a) {
                    var key = _a[0], value = _a[1];
                    headers.set(key, value);
                });
            }
            Object.entries(_this.tokens).forEach(function (_a) {
                var key = _a[0], value = _a[1];
                headers.set(key, value);
            });
            config.headers = headers;
            return config;
        });
        // 响应拦截器
        api.interceptors.response.use(function (response) {
            if (response.headers["set-cookie"]) {
                response.headers["set-cookie"].forEach(function (cookie) {
                    var cookieName = cookie.split("=")[0].trim();
                    _this.cookies.set(cookieName, cookie);
                });
                // 限制 cookie 数量
                if (_this.cookies.size > _this.maxCookies) {
                    var cookiesArray = Array.from(_this.cookies.entries());
                    _this.cookies = new Map(cookiesArray.slice(-_this.maxCookies));
                }
            }
            _this.config.auth.tokenKeys.forEach(function (key) {
                if (key in response.headers) {
                    _this.tokens[key] = response.headers[key];
                    console.log(key, _this.tokens[key]);
                }
            });
            return Promise.resolve(response);
        }, function (error) {
            return Promise.reject(error);
        });
    };
    return AxiosFactory;
}());

function description() {
    return "Based on axios. After simple configuration, you can easily use the existing http service";
}
var index = {
    description: description,
    AxiosFactory: AxiosFactory
};

export { AxiosFactory, index as default, description };
//# sourceMappingURL=index.es.js.map
